use crate::config::config::Config;
use crate::store::{
    ignorer::Ignorer,
    indexer::Indexer,
    nsobject::{self, NsObject},
};
use std::fs;
// use glob::glob;
use std::path::PathBuf;

pub struct AddArgs {
    /// List of files/dirs to add
    pub files: Vec<String>,
    /// Allow ignored files
    pub force: bool,
    /// Select all files
    pub all: bool,
}

pub fn exec(args: AddArgs, config: Config) {
    // Init ignorer
    let mut ignorer = Ignorer::new(config.get_root_unsafe());
    ignorer.active_nsignore(!args.force);

    // Init indexer
    let mut indexer = Indexer::new(config.get_root_unsafe());
    let _ = indexer.load();

    nsobject::init(config.get_root_unsafe());

    if args.all {
        return add_dir(config.get_root_unsafe(), &mut ignorer, &mut indexer);
    }

    for obj_to_add in args.files.iter() {
        let mut path_to_add = config.execution_path.clone();
        path_to_add.push(PathBuf::from(obj_to_add));

        if path_to_add.exists() {
            if path_to_add.is_dir() {
                indexer.index_dir(path_to_add.clone());
                add_dir(&path_to_add, &mut ignorer, &mut indexer);
            } else {
                indexer.index_file(path_to_add);
            }
        } else if NsObject::from_local_path(&path_to_add).exists() {
            indexer.index_file(path_to_add);
        } else {
            // try globbing
            // todo!()
            // else panic
            panic!("fatal: pathspec '{}' did not match any files", obj_to_add);
        }
    }

    dbg!(indexer.save());
    /*
       for all files
           if globbing
               take all match glob nextsyncignore
           else
               if dir
                   add dir
                   add all childs with nextsyncignore
               else
                   add files
    */
}

/// Add all files in a directory
///
/// # Parameters
/// * `dir`: the directory to add from
fn add_dir(dir: &PathBuf, ignorer: &mut Ignorer, indexer: &mut Indexer) {
    let entries = match fs::read_dir(dir) {
        Ok(entries) => entries,
        Err(err) => {
            eprintln!("Failed to read {} ({err})", dir.display());
            return;
        }
    };

    // Read locals files and folders
    for entry in entries {
        let entry = match entry {
            Ok(entry) => entry,
            Err(err) => {
                eprintln!("Failed to read entry {err}");
                continue;
            }
        };

        // ignore object if it is ns config file or nsignored
        if ignorer.should_ignore(&entry.path()) {
            continue;
        }

        if entry.path().is_dir() {
            indexer.index_dir(entry.path().clone());
            add_dir(&entry.path(), ignorer, indexer);
        } else {
            indexer.index_file(entry.path());
        }
    }
}
